// © 2022, CGVR (https://cgvr.informatik.uni-bremen.de/),
// Author: Andre Mühlenbrock (muehlenb@uni-bremen.de)

// Include this file only once when compiling:
#pragma once

// Include utility to open files during runtime:
#include <fstream>
#include <iostream>

// Include string:
#include <string>

/**
 * This is a wrapper class for shader programs in OpenGL.
 */

class Shader {
    unsigned int shaderProgram;

public:
    /**
     * Loads the source code of the given file paths, compiles it on the
     * GPU and stores the reference (=id, =name) to the program in the
     * attribute 'shaderProgram'.
     *
     * NOTE:
     * OpenGL shaders are usually not compiled when the C++ program
     * is compiled, but only when the program is started. I.e. we have
     * to read in from the source files at this point and then pass it
     * to OpenGL to be compiled on the graphics card.
     *
     * In recent OpenGL versions, it is also possible to compile shaders
     * before hand (so they don't have to be compiled every time you
     * start a game, but that's another topic ;-)).
     */
    Shader(std::string vertexShaderPath, std::string fragmentShaderPath);

    /**
     * Deletes the mesh and ensures that the memory on the GPU
     * is cleaned up.
     */
    ~Shader();

    /**
     * Binds the shader so that it will be used in future draw calls.
     */
    void bind();
};
